import { Injectable } from '@angular/core';
import { File, Transfer } from 'ionic-native';
import { ImageLoaderConfig } from "./image-loader-config";
export var ImageLoader = (function () {
    function ImageLoader(config) {
        var _this = this;
        this.config = config;
        /**
         * Indicates if the cache service is ready.
         * When the cache service isn't ready, images are loaded via browser instead.
         * @type {boolean}
         */
        this.isCacheReady = false;
        /**
         * Indicates if this service is initialized.
         * This service is initialized once all the setup is done.
         * @type {boolean}
         */
        this.isInit = false;
        if (window.location.protocol === 'http:' || window.location.protocol === 'https:') {
            // we are running on a browser, or using livereload
            // plugin will not function in this case
            this.isInit = true;
            this.throwWarning('You are running on a browser or using livereload, IonicImageLoader will not function, falling back to browser loading.');
        }
        else {
            document.addEventListener('deviceready', function () {
                _this.initCache();
            }, false);
        }
    }
    /**
     * Downloads an image via cordova-plugin-file-transfer
     * @param imageUrl {string} The remote URL of the image
     * @param localPath {string} The local path to store the image at
     * @returns {Promise<any>} Returns a promise that resolves when the download is complete, or rejects on error.
     */
    ImageLoader.prototype.downloadImage = function (imageUrl, localPath) {
        var transfer = new Transfer();
        return transfer.download(imageUrl, localPath);
    };
    /**
     * Gets the filesystem path of an image.
     * This will return the remote path if anything goes wrong or if the cache service isn't ready yet.
     * @param imageUrl {string} The remote URL of the image
     * @returns {Promise<string>} Returns a promise that will always resolve with an image URL
     */
    ImageLoader.prototype.getImagePath = function (imageUrl) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var getImage = function () {
                _this.getCachedImagePath(imageUrl)
                    .then(function (imagePath) {
                    resolve(imagePath);
                })
                    .catch(function () {
                    // image doesn't exist in cache, lets fetch it and save it
                    var localPath = cordova.file.cacheDirectory + _this.config.cacheDirectoryName + '/' + _this.createFileName(imageUrl);
                    _this.downloadImage(imageUrl, localPath)
                        .then(function () {
                        resolve(localPath);
                    })
                        .catch(function (e) {
                        reject();
                        _this.throwError(e);
                    });
                });
            };
            var check = function () {
                if (_this.isInit) {
                    if (_this.isCacheReady) {
                        getImage();
                    }
                    else {
                        _this.throwWarning('The cache system is not running. Images will be loaded by your browser instead.');
                        resolve(imageUrl);
                    }
                }
                else {
                    setTimeout(function () { return check(); }, 250);
                }
            };
            check();
        });
    };
    /**
     * Initialize the cache service
     * @param replace {boolean} Whether to replace the cache directory if it already exists
     */
    ImageLoader.prototype.initCache = function (replace) {
        var _this = this;
        if (!this.filePluginExists) {
            this.isInit = true;
            return;
        }
        this.cacheDirectoryExists
            .then(function () {
            // exists
            _this.isCacheReady = true;
            _this.isInit = true;
        })
            .catch(function () {
            // doesn't exist
            _this.createCacheDirectory(replace)
                .then(function () {
                _this.isCacheReady = true;
                _this.isInit = true;
            })
                .catch(function (e) {
                _this.throwError(e);
                _this.isInit = true;
            });
        });
    };
    /**
     * Get the local path of a previously cached image if exists
     * @param url {string} The remote URL of the image
     * @returns {Promise<string>} Returns a promise that resolves with the local path if exists, or rejects if doesn't exist
     */
    ImageLoader.prototype.getCachedImagePath = function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            // make sure cache is ready
            if (!_this.isCacheReady) {
                return reject();
            }
            // get file name
            var fileName = _this.createFileName(url);
            // get full path
            var dirPath = cordova.file.cacheDirectory + _this.config.cacheDirectoryName;
            // check if exists
            File.resolveLocalFilesystemUrl(dirPath + '/' + fileName)
                .then(function (fileEntry) {
                // file exists in cache
                resolve(fileEntry.nativeURL);
            })
                .catch(reject); // file doesn't exist
        });
    };
    /**
     * Throws a console error if debug mode is enabled
     * @param error {string} Error message
     */
    ImageLoader.prototype.throwError = function (error) {
        if (this.config.debugMode) {
            console.error('ImageLoader Error', error);
        }
    };
    /**
     * Throws a console warning if debug mode is enabled
     * @param error {string} Error message
     */
    ImageLoader.prototype.throwWarning = function (error) {
        if (this.config.debugMode) {
            console.warn('ImageLoader Warning', error);
        }
    };
    Object.defineProperty(ImageLoader.prototype, "filePluginExists", {
        /**
         * Check if file plugin exists
         * @returns {boolean} returns a boolean that indicates whether the plugin exists
         */
        get: function () {
            if (!cordova || !cordova.file) {
                this.throwWarning('Unable to find the cordova file plugin. ImageLoader will not cache images.');
                return false;
            }
            return true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImageLoader.prototype, "cacheDirectoryExists", {
        /**
         * Check if the cache directory exists
         * @returns {Promise<boolean|FileError>} Returns a promise that resolves if exists, and rejects if it doesn't
         */
        get: function () {
            return File.checkDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Creates the cache directory
     * @param replace {boolean} override directory if exists
     * @returns {Promise<DirectoryEntry|FileError>} Returns a promise that resolves if the directory was created, and rejects on error
     */
    ImageLoader.prototype.createCacheDirectory = function (replace) {
        if (replace === void 0) { replace = false; }
        return File.createDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName, replace);
    };
    /**
     * Creates a unique file name out of the URL
     * @param url {string} URL of the file
     * @returns {string} Unique file name
     */
    ImageLoader.prototype.createFileName = function (url) {
        // hash the url to get a unique file name
        return this.hashString(url).toString();
    };
    /**
     * Converts a string to a unique 32-bit int
     * @param string {string} string to hash
     * @returns {number} 32-bit int
     */
    ImageLoader.prototype.hashString = function (string) {
        var hash = 0;
        var char;
        if (string.length === 0)
            return hash;
        for (var i = 0; i < string.length; i++) {
            char = string.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash;
    };
    ImageLoader.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ImageLoader.ctorParameters = [
        { type: ImageLoaderConfig, },
    ];
    return ImageLoader;
}());
//# sourceMappingURL=image-loader.js.map